#!/bin/bash
#
# Script to convert a PEM private key file to a JWK object
#
# Requires the following command line utilities:
# - openssl
# - sed
# - xxd
# - base64
# - tr
# - basename

set +m
shopt -s lastpipe

if [ -z "$1" ] ; then
  >&2 echo "ERROR: Missing file to process"
  >&2 echo "Syntax: $0 PrivateKeyPEMFile > JWKFile" 
  exit 1
fi

if [ ! -f "$1" ] ; then
  >&2 echo "ERROR: Can't find $1"
  >&2 echo "Syntax: $0 PrivateKeyPEMFile > JWKFile" 
  exit 1
fi

openssl rsa -in $1 -text -noout | while read l ; do
  case $l in
    modulus:* )
      context='modulus';;

    publicExponent:* )
      context='publicExponent'
      e=$(printf "%06x" $(echo $l | sed 's/publicExponent: \([0-9]\+\) .*/\1/') | xxd -r -p | base64)
      ;;

    privateExponent:* )
      context='privateExponent';;

    prime1:* )
      context='prime1';;

    prime2:* )
      context='prime2';;

    exponent1:* )
      context='exponent1';;

    exponent2:* )
      context='exponent2';;

    coefficient:* )
      context='coefficient';;

    * )
      l=$(echo $l | sed 's/://g')	# Remove colons from line

      case $context in
	'modulus' )
	   n="$n$l"
	   ;;

	'prime1' )
	   p="$p$l"
	   ;;

	'prime2' )
	   q="$q$l"
	   ;;

	'privateExponent' )
	   d="$d$l"
	   ;;

	'exponent1' )
	   dp="$dp$l"
	   ;;

	'exponent2' )
	   dq="$dq$l"
	   ;;

	'coefficient' )
	   qi="$qi$l"
	   ;;

      esac
  esac
done

# strip leading 00, convert to binary, base64 encode, convert to base64url, strip trailing =
n=$(echo $n | sed 's/^00//' | xxd -r -p | base64 -w 0 | tr '+/' '-_' | sed 's/=*$//')
d=$(echo $d | sed 's/^00//' | xxd -r -p | base64 -w 0 | tr '+/' '-_' | sed 's/=*$//')
p=$(echo $p | sed 's/^00//' | xxd -r -p | base64 -w 0 | tr '+/' '-_' | sed 's/=*$//')
q=$(echo $q | sed 's/^00//' | xxd -r -p | base64 -w 0 | tr '+/' '-_' | sed 's/=*$//')
dp=$(echo $dp | sed 's/^00//' | xxd -r -p | base64 -w 0 | tr '+/' '-_' | sed 's/=*$//')
dq=$(echo $dq | sed 's/^00//' | xxd -r -p | base64 -w 0 | tr '+/' '-_' | sed 's/=*$//')
qi=$(echo $qi | sed 's/^00//' | xxd -r -p | base64 -w 0 | tr '+/' '-_' | sed 's/=*$//')

cat <<EOF
{
  "kty": "RSA",
  "use": "sig",
  "kid": "$(basename $1)",
  "n": "$n",
  "e": "$e",
  "d": "$d",
  "p": "$p",
  "q": "$q",
  "dp": "$dp",
  "dq": "$dq",
  "qi": "$qi"
}
EOF
